<?php

/**
 * @file
 * Contains the controller/route/error bindings, along with middleware.
 *
 * Controller resolution:
 * @see \Silex\ControllerResolver
 * @see \Symfony\Component\HttpKernel\Controller\ControllerResolver
 *
 * Error handler resolution:
 * @see \Silex\ExceptionListenerWrapper
 *
 * View resolution:
 * @see \Silex\ViewListenerWrapper
 *
 * Other resolutions:
 * @see \Silex\Provider\HttpKernelServiceProvider::subscribe()
 */

use demo\Controllers\BlogController;
use demo\Controllers\DelegatingController;
use demo\Controllers\ErrorController;
use demo\Controllers\EscapeController;
use demo\Controllers\FeedbackController;
use demo\Controllers\HomeController;
use demo\Controllers\StreamController;
use demo\Controllers\UserController;
use demo\Errors\GenericErrorHandler;
use demo\Errors\HttpErrorHandler;
use demo\Middleware\AfterAll;
use demo\Middleware\BeforeAll;
use demo\Middleware\Finish;
use demo\Views\JsonView;

//Request::setTrustedProxies(array('127.0.0.1'));

// Middleware must be callables, or registered in the container.
$app['mwAfterAll'] = function () { return new AfterAll(); };
$app['mwBeforeAll'] = function () { return new BeforeAll(); };

// This type of global configuration does not apply to mounted controllers,
// which have their own "global" configuration.

// Normal service with a method call. "Before": Triggered on KE::REQUEST.
$app->before('mwBeforeAll:next');
// Callable service. "After": triggered on KE::RESPONSE.
$app->after('mwAfterAll');
// Plain callable. "Finish": triggered on KE::TERMINATE.
$app->finish([Finish::class, 'handle']);

// Demo Twig.
$app->get('/',                  HomeController::class       . '::home')
  ->bind('homepage');

// Demo redirects and forwards.
$app->get('/home',              DelegatingController::class . '::redirectPath');
$app->get('/blogz',             DelegatingController::class . '::forwardPath');
$app->get('/all_blogs',         DelegatingController::class . '::forwardName');

// Demo typical entity listing routes with their modifiers.
$app->get('/blogs',             BlogController::class       . '::index')
  ->bind('blog_list');
$app->get('/blogs-json',        BlogController::class       . '::json');
$app->get('/blogs-json-view',   BlogController::class       . '::jsonView');
$app->get('/blog/{id}',         BlogController::class       . '::fifiAction')
  ->assert('id', '\d+')
  ->when("request.headers.get('User-Agent') matches '/firefox/i'");
$app->get('/blog/{id}',         BlogController::class       . '::show')
  ->assert('id', '\d+')
  ->value('id', 1)
  ->bind('blog_post');

// Demo POST request handling.
$app->post('/feedback',         FeedbackController::class   . '::feedbackAction');

// Demo escaping HTML and JSON
$app->get('/hello/{name}',      EscapeController::class     . '::html');
$app->get('/hello-json/{name}', EscapeController::class     . '::json');

// Demo streaming.
$app->get('/noise',             StreamController::class     . '::customStream');
$app->get('/pass',              StreamController::class     . '::fileStream');

// Demo error handling.
$app->get('err/http',           ErrorController::class      . '::errorHttp');
$app->get('err/base',           ErrorController::class      . '::errorBase');

// Demo param converters.
$app->get('/user/{user}',       UserController::class       . '::itemAction')
  ->convert('user', 'converter.user:convert');

// Register a view handler. They can also receive Request $request as 2nd arg,
// e.g. for basic content negotiation. But the callback_resolver service:
// - can not receive $app
// - unlike the controller_resolver, can only use standard callables/services
// - unlike the exception listener, can not use _invoke-able class names.
// So one way to get the app is to use it as a global (!).
// They are triggered on KE::VIEW.
$app->view([JsonView::class, 'handle']);

// Handlers are examined in order, and called in a chain with the result of the
// previous one. The last one must return a string or Response.
// They are triggered on KE::EXCEPTION.
$app->error([HttpErrorHandler::class, 'handle']);
// Handlers do not receive $app, so they have to be closures or instantiated
// here if they need it (or use the global $app like view handlers.
$app->error(new GenericErrorHandler($app));
